﻿using System.Text;
using WarmlyShip.Drawnings;

namespace WarmlyShip.CollectionGenericObjects;


public class StorageCollection<T> where T : DrawningShip
{

    readonly Dictionary<string, ICollectionGenericObjects<T>> _storages;

    public List<string> Keys => _storages.Keys.ToList();

    private readonly string _collectionKey = "CollectionStorage";

    
    private readonly string _separatorForKeyValue = "|";


    private readonly string _separatorItems = ";";


    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }

  
    public void AddCollection(string name, CollectionType collectionType)
    {
        if (_storages.ContainsKey(name) || collectionType == CollectionType.None)
        {
            return;
        }

        switch (collectionType)
        {
            case CollectionType.Massive:
                _storages[name] = new MassiveGenericObjects<T>();
                break;
            case CollectionType.List:
                _storages[name] = new ListGenericObjects<T>();
                break;
        }
    }

    public void DelCollection(string name)
    {
        if (_storages.ContainsKey(name) && name != null)
        {
            _storages.Remove(name);
        }
    }

    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            if (_storages.ContainsKey(name))
            {
                return _storages[name];
            }
            return null;
        }
    }
    public bool SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            return false;
        }

        if (File.Exists(filename))
        {
            File.Delete(filename);
        }

        using (StreamWriter writer = new StreamWriter(filename))
        {
            writer.Write(_collectionKey);
            foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
            {
                StringBuilder sb = new();

                sb.Append(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0)
                {
                    continue;
                }

                sb.Append(value.Key);
                sb.Append(_separatorForKeyValue);
                sb.Append(value.Value.GetCollectionType);
                sb.Append(_separatorForKeyValue);
                sb.Append(value.Value.MaxCount);
                sb.Append(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;
                    if (string.IsNullOrEmpty(data))
                    {
                        continue;
                    }

                    sb.Append(data);
                    sb.Append(_separatorItems);
                }

                writer.Write(sb);
            }

        }

        return true;
    }

    /// <summary>
    /// Загрузка информации по автомобилям в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
    public bool LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            return false;
        }

        using (StreamReader fs = File.OpenText(filename))
        {
            string str = fs.ReadLine();

            if (str == null || str.Length == 0)
            {
                return false;
            }

            if (!str.StartsWith(_collectionKey))
            {
                return false;
            }

            _storages.Clear();
            string strs = "";
            while ((strs = fs.ReadLine()) != null)
            {
                string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }

                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    return false;
                }

                collection.MaxCount = Convert.ToInt32(record[2]);

                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningShip() is T ship)
                    {
                        if (collection.Insert(ship) == -1)
                        {
                            return false;
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }
            return true;
        }
    }

    /// <summary>
    /// Создание коллекции по типа
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null
        };
    }
}
